@@ -1216,10 +1216,11 @@ try:
         handles = []
         handles.append(_render3D(ax, X, Y, Z, filled=filled, **kwargs))
         handles.append(
-            _render3D(ax, X, (2 * centre[1] - Y), Z, filled=filled, **kwargs)
+            _render3D(ax, X, (2 * centre[1] - Y), Z, filled=filled, pose=pose, **kwargs)
         )
 
         if ends and kwargs.get("filled", default=False):
+            # TODO: this should handle the pose argument, zdir can be a 3-tuple
             floor = Circle(centre[:2], radius, **kwargs)
             handles.append(ax.add_patch(floor))
             pathpatch_2d_to_3d(floor, z=height[0], zdir="z")
@@ -1457,14 +1458,16 @@ try:
 
         if hasattr(ax, "name"):
             # handle the case of some kind of matplotlib Axes
-            return 3 if ax.name == "3d" else 2
+            ret = 3 if ax.name == "3d" else 2
         else:
             # handle the case of Animate objects pretending to be Axes
             classname = ax.__class__.__name__
             if classname == "Animate":
-                return 3
+                ret = 3
             elif classname == "Animate2":
-                return 2
+                ret = 2
+        # print("_axes_dimensions ", ax, ret)
+        return ret
 
     def axes_get_limits(ax: plt.Axes) -> NDArray:
         return np.r_[ax.get_xlim(), ax.get_ylim()]
@@ -1495,6 +1498,7 @@ try:
         dimensions: int,
         projection: Optional[str] = "ortho",
         autoscale: Optional[bool] = True,
+        new: Optional[bool] = False,
     ) -> Union[plt.Axes, Axes3D]:
         """
         Axis creation logic
@@ -1505,6 +1509,8 @@ try:
         :type dimensions: int
         :param projection: 3D projection type, defaults to 'ortho'
         :type projection: str, optional
+        :param new: create a new figure, defaults to False
+        :type new: bool
         :return: axes to draw in
         :rtype: Axes3DSubplot or AxesSubplot
 
@@ -1514,6 +1520,9 @@ try:
         If the dimensions do not match, or no figure/axes currently exist,
         then ``plt.axes()`` is called to create one.
 
+        If ``new`` is True then a new 3D axes is created regardless of whether the
+        current axis is 3D.
+
         Used by all plot_xxx() functions in this module.
         """
 
@@ -1533,21 +1542,19 @@ try:
                 if naxes > 0:
                     ax = plt.gca()  # get current axes
                     # print(f"ax has {_axes_dimensions(ax)} dimensions")
-                    if _axes_dimensions(ax) == dimensions:
+                    if _axes_dimensions(ax) == dimensions and not new:
                         return ax
             # otherwise it doesnt exist or dimension mismatch, create new axes
-            # print("create new axes")
         else:
-            # print("no figs present, ax given")
+            # print("ax given", ax)
             # axis was given
 
             if _axes_dimensions(ax) == dimensions:
                 # print("use existing axes")
                 return ax
-            # mismatch in dimensions, create new axes
-        # print('create new axes')
+            # print("mismatch in dimensions, create new axes")
+        # print("create new axes")
         plt.figure()
-        # no axis specified
         if dimensions == 2:
             ax = plt.axes()
             if autoscale:
@@ -1556,7 +1563,7 @@ try:
             ax = plt.axes(projection="3d", proj_type=projection)
 
         plt.sca(ax)
-        plt.axes(ax)
+        # plt.axes(ax)
 
         return ax
 
@@ -1566,6 +1573,7 @@ try:
         equal: Optional[bool] = True,
         grid: Optional[bool] = False,
         labels: Optional[bool] = True,
+        new: Optional[bool] = False,
     ) -> plt.Axes:
         """
         Create 2D plot area
@@ -1589,9 +1597,11 @@ try:
 
         :seealso: :func:`plotvol3`, :func:`expand_dims`
         """
+        ax = axes_logic(ax, 2, new=new)
+
         dims = expand_dims(dim, 2)
-        if ax is None:
-            ax = plt.subplot()
+        # if ax is None:
+        #     ax = plt.subplot()
         ax.axis(dims)
         if labels:
             ax.set_xlabel("X")
@@ -1614,6 +1624,7 @@ try:
         grid: Optional[bool] = False,
         labels: Optional[bool] = True,
         projection: Optional[str] = "ortho",
+        new: Optional[bool] = False,
     ) -> Axes3D:
         """
         Create 3D plot volume
@@ -1638,7 +1649,7 @@ try:
         :seealso: :func:`plotvol2`, :func:`expand_dims`
         """
         # create an axis if none existing
-        ax = axes_logic(ax, 3, projection=projection)
+        ax = axes_logic(ax, 3, projection=projection, new=new)
 
         if dim is None:
             ax.autoscale(True)
