@@ -13,13 +13,14 @@ tuple, numpy array, numpy row vector or numpy column vector.
 
 import math
 import numpy as np
-from spatialmath import base
+from spatialmath.base.types import *
+from spatialmath.base.argcheck import getvector, isvector
+from spatialmath.base.vectors import iszerovec, unitvec_norm
+
+# from spatialmath.base.symbolic import issymbol
+# from spatialmath.base.transforms3d import transl
+# from spatialmath.base.transforms2d import transl2
 
-# from spatialmath.base import vectors as vec
-# from spatialmath.base import transforms2d as t2d
-# from spatialmath.base import transforms3d as t3d
-# from spatialmath.base import argcheck
-# from spatialmath.base import symbolic as sym
 
 try:  # pragma: no cover
     # print('Using SymPy')
@@ -32,7 +33,18 @@ except ImportError:  # pragma: no cover
 
 _eps = np.finfo(np.float64).eps
 
+
 # ---------------------------------------------------------------------------------------#
+@overload
+def r2t(R: SO2Array, check: bool = False) -> SE2Array:
+    ...
+
+
+@overload
+def r2t(R: SO3Array, check: bool = False) -> SE3Array:
+    ...
+
+
 def r2t(R, check=False):
     """
     Convert SO(n) to SE(n)
@@ -89,7 +101,17 @@ def r2t(R, check=False):
 
 
 # ---------------------------------------------------------------------------------------#
-def t2r(T, check=False):
+@overload
+def t2r(T: SE2Array, check: bool = False) -> SO2Array:
+    ...
+
+
+@overload
+def t2r(T: SE3Array, check: bool = False) -> SO3Array:
+    ...
+
+
+def t2r(T: SEnArray, check: bool = False) -> SOnArray:
     """
     Convert SE(n) to SO(n)
 
@@ -137,10 +159,24 @@ def t2r(T, check=False):
     return R
 
 
+a = t2r(np.eye(4, dtype="float"))
+
+b = t2r(np.eye(3))
+
 # ---------------------------------------------------------------------------------------#
 
 
-def tr2rt(T, check=False):
+@overload
+def tr2rt(T: SE2Array, check=False) -> Tuple[SO2Array, R2]:
+    ...
+
+
+@overload
+def tr2rt(T: SE3Array, check=False) -> Tuple[SO3Array, R3]:
+    ...
+
+
+def tr2rt(T: SEnArray, check=False) -> Tuple[SOnArray, Rn]:
     """
     Convert SE(n) to SO(n) and translation
 
@@ -184,12 +220,22 @@ def tr2rt(T, check=False):
     else:
         raise ValueError("T must be an SE2 or SE3 homogeneous transformation matrix")
 
-    return [R, t]
+    return (R, t)
 
 
 # ---------------------------------------------------------------------------------------#
 
 
+@overload
+def rt2tr(R: SO2Array, t: ArrayLike2, check=False) -> SE2Array:
+    ...
+
+
+@overload
+def rt2tr(R: SO3Array, t: ArrayLike3, check=False) -> SE3Array:
+    ...
+
+
 def rt2tr(R, t, check=False):
     """
     Convert SO(n) and translation to SE(n)
@@ -220,7 +266,7 @@ def rt2tr(R, t, check=False):
 
     :seealso: rt2m, tr2rt, r2t
     """
-    t = base.getvector(t, dim=None, out="array")
+    t = getvector(t, dim=None, out="array")
     if not isinstance(R, np.ndarray):
         raise ValueError("Rotation matrix not a NumPy array")
     if R.shape[0] != t.shape[0]:
@@ -230,17 +276,16 @@ def rt2tr(R, t, check=False):
 
     if R.dtype == "O":
         if R.shape == (2, 2):
-            T = np.pad(R, ((0, 1), (0, 1)), 'constant')
+            T = np.pad(R, ((0, 1), (0, 1)), "constant")  # type: ignore
             T[:2, 2] = t
             T[2, 2] = 1
         elif R.shape == (3, 3):
-            T = np.pad(R, ((0, 1), (0, 1)), 'constant')
+            T = np.pad(R, ((0, 1), (0, 1)), "constant")  # type: ignore
             T[:3, 3] = t
             T[3, 3] = 1
         else:
             raise ValueError("R must be an SO2 or SO3 rotation matrix")
     else:
-
         if R.shape == (2, 2):
             T = np.eye(3)
             T[:2, :2] = R
@@ -258,7 +303,7 @@ def rt2tr(R, t, check=False):
 # ---------------------------------------------------------------------------------------#
 
 
-def Ab2M(A, b):
+def Ab2M(A: np.ndarray, b: np.ndarray) -> np.ndarray:
     """
     Pack matrix and vector to matrix
 
@@ -285,7 +330,7 @@ def Ab2M(A, b):
 
     :seealso: rt2tr, tr2rt, r2t
     """
-    b = base.getvector(b, dim=None, out="array")
+    b = getvector(b, dim=None, out="array")
     if not isinstance(A, np.ndarray):
         raise ValueError("Rotation matrix not a NumPy array")
     if A.shape[0] != b.shape[0]:
@@ -308,7 +353,7 @@ def Ab2M(A, b):
 # ======================= predicates
 
 
-def isR(R, tol=100):
+def isR(R: NDArray, tol: float = 100) -> bool:  # -> TypeGuard[SOnArray]:
     r"""
     Test if matrix belongs to SO(n)
 
@@ -331,13 +376,13 @@ def isR(R, tol=100):
 
     :seealso: isrot2, isrot
     """
-    return (
+    return bool(
         np.linalg.norm(R @ R.T - np.eye(R.shape[0])) < tol * _eps
         and np.linalg.det(R @ R.T) > 0
     )
 
 
-def isskew(S, tol=10):
+def isskew(S: NDArray, tol: float = 10) -> bool:  # -> TypeGuard[sonArray]:
     r"""
     Test if matrix belongs to so(n)
 
@@ -361,10 +406,10 @@ def isskew(S, tol=10):
 
     :seealso: isskewa
     """
-    return np.linalg.norm(S + S.T) < tol * _eps
+    return bool(np.linalg.norm(S + S.T) < tol * _eps)
 
 
-def isskewa(S, tol=10):
+def isskewa(S: NDArray, tol: float = 10) -> bool:  # -> TypeGuard[senArray]:
     r"""
     Test if matrix belongs to se(n)
 
@@ -389,12 +434,12 @@ def isskewa(S, tol=10):
 
     :seealso: isskew
     """
-    return np.linalg.norm(S[0:-1, 0:-1] + S[0:-1, 0:-1].T) < tol * _eps and np.all(
+    return bool(np.linalg.norm(S[0:-1, 0:-1] + S[0:-1, 0:-1].T) < tol * _eps) and all(
         S[-1, :] == 0
     )
 
 
-def iseye(S, tol=10):
+def iseye(S: NDArray, tol: float = 10) -> bool:
     """
     Test if matrix is identity
 
@@ -405,8 +450,8 @@ def iseye(S, tol=10):
     :return: whether matrix is a proper skew-symmetric matrix
     :rtype: bool
 
-    Check if matrix is an identity matrix. We test that the trace tom row is zero
-    We check that the norm of the residual is less than ``tol * eps``.
+    Check if matrix is an identity matrix. 
+    We check that the sum of the absolute value of the residual is less than ``tol * eps``.
 
     .. runblock:: pycon
 
@@ -420,10 +465,20 @@ def iseye(S, tol=10):
     s = S.shape
     if len(s) != 2 or s[0] != s[1]:
         return False  # not a square matrix
-    return np.linalg.norm(S - np.eye(s[0])) < tol * _eps
+    return bool(np.abs(S - np.eye(s[0])).sum() < tol * _eps)
 
 
 # ---------------------------------------------------------------------------------------#
+@overload
+def skew(v: float) -> se2Array:
+    ...
+
+
+@overload
+def skew(v: ArrayLike3) -> se3Array:
+    ...
+
+
 def skew(v):
     r"""
     Create skew-symmetric metrix from vector
@@ -450,19 +505,38 @@ def skew(v):
         - This is the inverse of the function ``vex()``.
         - These are the generator matrices for the Lie algebras so(2) and so(3).
 
-    :seealso: :func:`vex`, :func:`skewa`
+    :seealso: :func:`vex` :func:`skewa`
     :SymPy: supported
     """
-    v = base.getvector(v, None, "sequence")
+    v = getvector(v, None, "sequence")
     if len(v) == 1:
-        return np.array([[0, -v[0]], [v[0], 0]])
+        # fmt: off
+        return np.array([
+            [0.0,  -v[0]], 
+            [v[0],  0.0]
+        ])  # type: ignore
+        # fmt: on
     elif len(v) == 3:
-        return np.array([[0, -v[2], v[1]], [v[2], 0, -v[0]], [-v[1], v[0], 0]])
+        # fmt: off
+        return np.array([
+            [ 0,    -v[2],  v[1]],
+            [ v[2],  0,    -v[0]],
+            [-v[1],  v[0],  0]
+        ])  # type: ignore
+        # fmt: on
     else:
         raise ValueError("argument must be a 1- or 3-vector")
 
 
 # ---------------------------------------------------------------------------------------#
+@overload
+def vex(s: so2Array, check: bool = False) -> R1:
+    ...
+
+
+@overload
+def vex(s: so3Array, check: bool = False) -> R3:
+    ...
 
 
 def vex(s, check=False):
@@ -500,7 +574,7 @@ def vex(s, check=False):
         - The function takes the mean of the two elements that correspond to each unique
           element of the matrix.
 
-    :seealso: :func:`skew`, :func:`vexa`
+    :seealso: :func:`skew` :func:`vexa`
     :SymPy: supported
     """
     if s.shape == (3, 3):
@@ -514,9 +588,17 @@ def vex(s, check=False):
 
 
 # ---------------------------------------------------------------------------------------#
+@overload
+def skewa(v: ArrayLike3) -> se2Array:
+    ...
 
 
-def skewa(v):
+@overload
+def skewa(v: ArrayLike6) -> se3Array:
+    ...
+
+
+def skewa(v: Union[ArrayLike3, ArrayLike6]) -> Union[se2Array, se3Array]:
     r"""
     Create augmented skew-symmetric metrix from vector
 
@@ -543,11 +625,11 @@ def skewa(v):
         - These are the generator matrices for the Lie algebras se(2) and se(3).
         - Map twist vectors in 2D and 3D space to se(2) and se(3).
 
-    :seealso: :func:`vexa`, :func:`skew`
+    :seealso: :func:`vexa` :func:`skew`
     :SymPy: supported
     """
 
-    v = base.getvector(v, None)
+    v = getvector(v, None)
     if len(v) == 3:
         omega = np.zeros((3, 3), dtype=v.dtype)
         omega[:2, :2] = skew(v[2])
@@ -562,7 +644,17 @@ def skewa(v):
         raise ValueError("expecting a 3- or 6-vector")
 
 
-def vexa(Omega, check=False):
+@overload
+def vexa(Omega: se2Array, check: bool = False) -> R3:
+    ...
+
+
+@overload
+def vexa(Omega: se3Array, check: bool = False) -> R6:
+    ...
+
+
+def vexa(Omega: senArray, check: bool = False) -> Union[R3, R6]:
     r"""
     Convert skew-symmetric matrix to vector
 
@@ -597,62 +689,18 @@ def vexa(Omega, check=False):
         - The function takes the mean of the two elements that correspond to each unique
           element of the matrix.
 
-    :seealso: :func:`skewa`, :func:`vex`
+    :seealso: :func:`skewa` :func:`vex`
     :SymPy: supported
     """
     if Omega.shape == (4, 4):
-        return np.hstack((base.transl(Omega), vex(t2r(Omega), check=check)))
+        return np.hstack((Omega[:3, 3], vex(Omega[:3, :3], check=check)))
     elif Omega.shape == (3, 3):
-        return np.hstack((base.transl2(Omega), vex(t2r(Omega), check=check)))
+        return np.hstack((Omega[:2, 2], vex(Omega[:2, :2], check=check)))
     else:
         raise ValueError("expecting a 3x3 or 4x4 matrix")
 
 
-def rodrigues(w, theta=None):
-    r"""
-    Rodrigues' formula for rotation
-
-    :param w: rotation vector
-    :type w: array_like(3) or array_like(1)
-    :param θ: rotation angle
-    :type θ: float or None
-    :return: SO(n) matrix
-    :rtype: ndarray(2,2) or ndarray(3,3)
-
-    Compute Rodrigues' formula for a rotation matrix given a rotation axis
-    and angle.
-
-    .. math::
-
-        \mat{R} = \mat{I}_{3 \times 3} + \sin \theta \skx{\hat{\vec{v}}} + (1 - \cos \theta) \skx{\hat{\vec{v}}}^2
-
-    .. runblock:: pycon
-
-        >>> from spatialmath.base import *
-        >>> rodrigues([1, 0, 0], 0.3)
-        >>> rodrigues([0.3, 0, 0])
-        >>> rodrigues(0.3)   # 2D version
-
-    """
-    w = base.getvector(w)
-    if base.iszerovec(w):
-        # for a zero so(n) return unit matrix, theta not relevant
-        if len(w) == 1:
-            return np.eye(2)
-        else:
-            return np.eye(3)
-    if theta is None:
-        w, theta = base.unitvec_norm(w)
-
-    skw = skew(w)
-    return (
-        np.eye(skw.shape[0])
-        + math.sin(theta) * skw
-        + (1.0 - math.cos(theta)) * skw @ skw
-    )
-
-
-def h2e(v):
+def h2e(v: NDArray) -> NDArray:
     """
     Convert from homogeneous to Euclidean form
 
@@ -683,13 +731,16 @@ def h2e(v):
         # dealing with matrix
         return v[:-1, :] / v[-1, :][np.newaxis, :]
 
-    elif base.isvector(v):
+    elif isvector(v):
         # dealing with shape (N,) array
-        v = base.getvector(v, out="col")
+        v = getvector(v, out="col")
         return v[0:-1] / v[-1]
 
+    else:
+        raise ValueError("bad type")
 
-def e2h(v):
+
+def e2h(v: NDArray) -> NDArray:
     """
     Convert from Euclidean to homogeneous form
 
@@ -719,13 +770,16 @@ def e2h(v):
         # dealing with matrix
         return np.vstack([v, np.ones((1, v.shape[1]))])
 
-    elif base.isvector(v):
+    elif isvector(v):
         # dealing with shape (N,) array
-        v = base.getvector(v, out="col")
+        v = getvector(v, out="col")
         return np.vstack((v, 1))
 
+    else:
+        raise ValueError("bad type")
 
-def homtrans(T, p):
+
+def homtrans(T: SEnArray, p: np.ndarray) -> np.ndarray:
     r"""
     Apply a homogeneous transformation to a Euclidean vector
 
@@ -758,7 +812,7 @@ def homtrans(T, p):
           then the points are defined with respect to frame {B} and are transformed to be
           with respect to frame {A}.
 
-    :seealso: :func:`e2h`, :func:`h2e`
+    :seealso: :func:`e2h` :func:`h2e`
     """
     p = e2h(p)
     if p.shape[0] != T.shape[0]:
@@ -767,7 +821,7 @@ def homtrans(T, p):
     return h2e(T @ p)
 
 
-def det(m):
+def det(m: np.ndarray) -> float:
     """
     Determinant of matrix
 
@@ -787,7 +841,7 @@ def det(m):
 
     :SymPy: supported
     """
-    if m.dtype.kind == "O":
+    if m.dtype.kind == "O" and _symbolics:
         return Matrix(m).det()
     else:
         return np.linalg.det(m)
@@ -796,10 +850,11 @@ def det(m):
 if __name__ == "__main__":  # pragma: no cover
     import pathlib
 
-    print(e2h((1, 2, 3)))
-    print(h2e((1, 2, 3)))
     exec(
         open(
-            pathlib.Path(__file__).parent.absolute() / "test" / "test_transformsNd.py"
+            pathlib.Path(__file__).parent.parent.parent.absolute()
+            / "tests"
+            / "base"
+            / "test_transformsNd.py"
         ).read()
     )  # pylint: disable=exec-used
