@@ -15,7 +15,9 @@ from matplotlib.patches import PathPatch
 from matplotlib.transforms import Affine2D
 import numpy as np
 
-from spatialmath import base, SE2
+from spatialmath import SE2
+import spatialmath.base as smb
+from spatialmath.base import plot_ellipse
 from spatialmath.base.types import (
     Points2,
     Optional,
@@ -50,8 +52,7 @@ class Line2:
     """
 
     def __init__(self, line: ArrayLike3):
-
-        self.line = base.getvector(line, 3)
+        self.line = smb.getvector(line, 3)
 
     @classmethod
     def Join(cls, p1: ArrayLike2, p2: ArrayLike2) -> Self:
@@ -66,10 +67,10 @@ class Line2:
         The points can be given in Euclidean or homogeneous form.
         """
 
-        p1 = base.getvector(p1)
+        p1 = smb.getvector(p1)
         if len(p1) == 2:
             p1 = np.r_[p1, 1]
-        p2 = base.getvector(p2)
+        p2 = smb.getvector(p2)
         if len(p2) == 2:
             p2 = np.r_[p2, 1]
 
@@ -118,7 +119,7 @@ class Line2:
 
         :param kwargs: arguments passed to Matplotlib ``pyplot.plot``
         """
-        base.plot_homline(self.line, **kwargs)
+        smb.plot_homline(self.line, **kwargs)
 
     def intersect(self, other: Line2, tol: float = 10) -> R3:
         """
@@ -146,7 +147,7 @@ class Line2:
         :return: True if point lies in the line
         :rtype: bool
         """
-        p = base.getvector(p)
+        p = smb.getvector(p)
         if len(p) == 2:
             p = np.r_[p, 1]
         return abs(np.dot(self.line, p)) < tol * _eps
@@ -167,10 +168,10 @@ class Line2:
         Tests whether the line intersects the line segment defined by endpoints
         ``p1`` and ``p2`` which are given in Euclidean or homogeneous form.
         """
-        p1 = base.getvector(p1)
+        p1 = smb.getvector(p1)
         if len(p1) == 2:
             p1 = np.r_[p1, 1]
-        p2 = base.getvector(p2)
+        p2 = smb.getvector(p2)
         if len(p2) == 2:
             p2 = np.r_[p2, 1]
 
@@ -191,7 +192,6 @@ class Line2:
         pass
 
     def points_join(self):
-
         pass
 
     def intersect_polygon___line(self):
@@ -278,6 +278,9 @@ class Polygon2:
         """
         return f"Polygon2 with {len(self.path)} vertices"
 
+    def __repr__(self) -> str:
+        return str(self)
+
     def __len__(self) -> int:
         """
         Number of vertices in polygon
@@ -411,10 +414,35 @@ class Polygon2:
         A Matplotlib Patch is created with the passed options ``**kwargs`` and
         added to the axes.
 
+        Examples::
+
+            >>> from spatialmath.base import plotvol2, plot_polygon
+            >>> plotvol2(5)
+            >>> p = Polygon2([(1, 2), (3, 2), (2, 4)])
+            >>> p.plot(fill=False)
+            >>> p.plot(facecolor="g", edgecolor="none")  # green filled triangle
+
+        .. plot::
+
+            from spatialmath import Polygon2
+            from spatialmath.base import plotvol2
+            p = Polygon2([(1, 2), (3, 2), (2, 4)])
+            plotvol2(5)
+            p.plot(fill=False)
+
+        .. plot::
+
+            from spatialmath import Polygon2
+            from spatialmath.base import plotvol2
+            p = Polygon2([(1, 2), (3, 2), (2, 4)])
+            plotvol2(5)
+            p.plot(facecolor="g", edgecolor="none")  # green filled triangle
+
+
         :seealso: :meth:`animate` :func:`matplotlib.PathPatch`
         """
         self.patch = PathPatch(self.path, **kwargs)
-        ax = base.axes_logic(ax, 2)
+        ax = smb.axes_logic(ax, 2)
         ax.add_patch(self.patch)
         plt.draw()
         self.kwargs = kwargs
@@ -528,7 +556,7 @@ class Polygon2:
         c = self.centroid()
         dmax = -np.inf
         for vertex in self.path.vertices:
-            d = base.norm(vertex - c)
+            d = smb.norm(vertex - c)
             dmax = max(dmax, d)
         return dmax
 
@@ -558,12 +586,12 @@ class Polygon2:
                 if other.intersect_segment(p1, p2):
                     return True
             return False
-        elif base.islistof(other, Polygon2):
+        elif smb.islistof(other, Polygon2):
             for polygon in cast(List[Polygon2], other):
                 if self.path.intersects_path(polygon.path, filled=True):
                     return True
             return False
-        elif base.islistof(other, Line2):
+        elif smb.islistof(other, Line2):
             for line in cast(List[Line2], other):
                 for p1, p2 in self.edges():
                     # test each edge segment against the line
@@ -641,65 +669,474 @@ class Polygon2:
 
 
 class Ellipse:
-    @classmethod
-    def Matrix(cls, E: NDArray, centre: ArrayLike2 = (0, 0)):
-        pass
+    def __init__(
+        self,
+        radii: Optional[ArrayLike2] = None,
+        E: Optional[NDArray] = None,
+        centre: ArrayLike2 = (0, 0),
+        theta: Optional[float] = None,
+    ):
+        r"""
+        Create an ellipse
+
+        :param radii: radii of ellipse, defaults to None
+        :type radii: arraylike(2), optional
+        :param E: 2x2 matrix describing ellipse, defaults to None
+        :type E: ndarray(2,2), optional
+        :param centre: centre of ellipse, defaults to (0, 0)
+        :type centre: arraylike(2), optional
+        :param theta: orientation of ellipse, defaults to None
+        :type theta: float, optional
+        :raises ValueError: bad parameters
+
+        The ellipse shape can be specified by ``radii`` and ``theta`` or by a
+        symmetric 2x2 matrix ``E``.
+
+        Internally the ellipse is represented by a symmetric matrix :math:`\mat{E} \in \mathbb{R}^{2\times 2}`
+        and its centre coordinate :math:`\vec{x}_0 \in \mathbb{R}^2` such that
+
+        .. math::
+
+            (\vec{x} - \vec{x}_0)^{\top} \mat{E} \, (\vec{x} - \vec{x}_0) = 1
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> import numpy as np
+            >>> Ellipse(radii=(1,2), theta=0)
+            >>> Ellipse(E=np.array([[1, 1], [1, 2]]))
+
+        """
+        if E is not None:
+            if not smb.ismatrix(E, (2, 2)):
+                raise ValueError("matrix must be 2x2")
+            if not np.allclose(E, E.T):
+                raise ValueError("matrix must be symmetric")
+            if np.linalg.det(E) <= 0:
+                raise ValueError("determinant of E must be > 0 for an ellipse")
+            self._E = E
+        elif radii is not None:
+            M = np.array(
+                [[np.cos(theta), np.sin(theta)], [np.sin(theta), -np.cos(theta)]]
+            )
+            self._E = M.T @ np.diag([radii[0] ** (-2), radii[1] ** (-2)]) @ M
+        else:
+            raise ValueError("must specify radii or E")
+
+        self._centre = centre
 
     @classmethod
-    def Parameters(cls, centre: ArrayLike2 = (0, 0), radii=(1, 1), orientation=0):
-        xc, yc = centre
-        alpha = 1.0 / radii[0]
-        beta = 1.0 / radii[1]
-        gamma = 0
+    def Polynomial(cls, e: ArrayLike, p: Optional[ArrayLike2] = None) -> Self:
+        r"""
+        Create an ellipse from polynomial
 
-        e0 = alpha
-        e1 = beta
-        e2 = 2 * gamma
-        e3 = -2 * (alpha * xc + gamma * yc)
-        e4 = -2 * (beta * yc + gamma * xc)
-        e5 = alpha * xc**2 + beta * yc**2 + 2 * gamma * xc * yc - 1
+        :param e: polynomial coeffients :math:`e` or :math:`\eta`
+        :type e: arraylike(4) or arraylike(5)
+        :param p: point to set scale
+        :type p: array_like(2), optional
+        :return: an ellipse instance
+        :rtype: Ellipse
 
-        self.e0 = e1 / e0
-        self.e1 = e2 / e0
-        self.e2 = e3 / e0
-        self.e3 = e4 / e0
-        self.e4 = e5 / e0
+        An ellipse can be specified by a polynomial :math:`\vec{e} \in \mathbb{R}^6`
+
+        .. math::
+
+            e_0 x^2 + e_1 y^2 + e_2 xy + e_3 x + e_4 y + e_5 = 0
+
+        or :math:`\vec{\epsilon} \in \mathbb{R}^5` where the leading coefficient is
+        implicitly one
+
+        .. math::
+
+            x^2 + \epsilon_1 y^2 + \epsilon_2 xy + \epsilon_3 x + \epsilon_4 y + \epsilon_5 = 0
+
+        In this latter case, position, orientation and aspect ratio of the
+        ellipse will be correct, but the overall scale of the ellipse is not
+        determined.  To correct this, we can pass in a single point ``p`` that
+        we know lies on the perimeter of the ellipse.
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> Ellipse.Polynomial([0.625, 0.625, 0.75, -6.75, -7.25, 24.625])
+
+        :seealso: :meth:`polynomial`
+        """
+        e = np.array(e)
+        if len(e) == 5:
+            e = np.insert(e, 0, 1.0)
+
+        a = e[0]
+        b = e[1]
+        c = e[2] / 2
+
+        # fmt: off
+        E = np.array([
+            [a, c],
+            [c, b],
+        ])
+        # fmt: on
+
+        # solve for the centre
+        centre = np.linalg.lstsq(-2 * E, e[3:5], rcond=None)[0]
+
+        if p is not None:
+            # point was passed in, use this to set the scale
+            p = smb.getvector(p, 2) - centre
+            s = p @ E @ p
+            E /= s
+
+        return cls(E=E, centre=centre)
 
     @classmethod
-    def Polynomial(cls, e: ArrayLike):
-        pass
+    def FromPoints(cls, p) -> Self:
+        """
+        Create an equivalent ellipse from a set of interior points
+
+        :param p: a set of 2D interior points
+        :type p: ndarray(2,N)
+        :return: an ellipse instance
+        :rtype: Ellipse
+
+        Computes the ellipse that has the same inertia as the set of points.
+
+        :seealso: :meth:`FromPerimeter`
+        """
+        # compute the moments
+        m00 = smb.mpq_point(p, 0, 0)
+        m10 = smb.mpq_point(p, 1, 0)
+        m01 = smb.mpq_point(p, 0, 1)
+        xc = np.c_[m10, m01] / m00
+
+        # compute the central second moments
+        x0 = p - xc.T
+        u20 = smb.mpq_point(x0, 2, 0)
+        u02 = smb.mpq_point(x0, 0, 2)
+        u11 = smb.mpq_point(x0, 1, 1)
+
+        # compute inertia tensor and ellipse matrix
+        J = np.array([[u20, u11], [u11, u02]])
+        E = m00 / 4 * np.linalg.inv(J)
+        centre = xc.flatten()
+
+        return cls(E=E, centre=centre)
+
+    @classmethod
+    def FromPerimeter(cls, p: Points2) -> Self:
+        """
+        Create an ellipse that fits a set of perimeter points
+
+        :param p: a set of 2D perimeter points
+        :type p: ndarray(2,N)
+        :return: an ellipse instance
+        :rtype: Ellipse
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> import numpy as np
+            >>> eref = Ellipse(radii=(1, 2), theta=np.pi / 4, centre=[3, 4])
+            >>> perim = eref.points()
+            >>> print(perim.shape)
+            >>> Ellipse.FromPerimeter(perim)
+
+        :seealso: :meth:`points`
+        """
+        A = []
+        b = []
+        for x, y in p.T:
+            A.append([y**2, x * y, x, y, 1])
+            b.append(-(x**2))
+        # solve for polynomial coefficients eta such that
+        #  x^2 + eta[0] y^2 + eta[1] xy + eta[2] x + eta[3] y + eta[4] = 0
+        e = np.linalg.lstsq(A, b, rcond=None)[0]
+
+        # create ellipse from the polynomial, using one point to set scale
+        return cls.Polynomial(e, p[:, 0])
 
     def __str__(self) -> str:
-        return f"Ellipse({self.e0}, {self.e1}, {self.e2}, {self.e3}, {self.e4})"
+        return f"Ellipse(radii={self.radii}, centre={self.centre}, theta={self.theta})"
 
+    def __repr__(self) -> str:
+        return f"Ellipse(radii={self.radii}, centre={self.centre}, theta={self.theta})"
+
+    @property
     def E(self):
-        # return 3x3 ellipse matrix
-        pass
+        r"""
+        Return ellipse matrix
 
+        :return: ellipse matrix
+        :rtype: ndarray(2,2)
+
+        The symmetric matrix :math:`\mat{E} \in \mathbb{R}^{2\times 2}` determines the radii and
+        the orientation of the ellipse
+
+        .. math::
+
+            (\vec{x} - \vec{x}_0)^{\top} \mat{E} \, (\vec{x} - \vec{x}_0) = 1
+
+        :seealso: :meth:`centre` :meth:`theta` :meth:`radii`
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.E
+        """
+        # return 2x2 ellipse matrix
+        return self._E
+
+    @property
     def centre(self) -> R2:
+        """
+        Return ellipse centre
+
+        :return: centre of the ellipse
+        :rtype: ndarray(2)
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.centre
+
+        :seealso: :meth:`radii` :meth:`theta` :meth:`E`
+        """
         # return centre
-        pass
+        return self._centre
 
+    @property
+    def radii(self) -> R2:
+        """
+        Return radii of the ellipse
+
+        :return: radii of the ellipse
+        :rtype: ndarray(2)
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.radii
+
+        :seealso: :meth:`centre` :meth:`theta` :meth:`E`
+        """
+        return np.linalg.eigvals(self.E) ** (-0.5)
+
+    @property
+    def theta(self) -> float:
+        """
+        Return orientation of ellipse
+
+        :return: orientation in radians, in the interval [-pi, pi)
+        :rtype: float
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.theta
+
+        :seealso: :meth:`centre` :meth:`radii` :meth:`E`
+        """
+        e, x = np.linalg.eigh(self.E)
+        # major axis is second column
+        return np.arctan(x[1, 1] / x[0, 1])
+
+    @property
+    def area(self) -> float:
+        """
+        Area of ellipse
+
+        :return: area
+        :rtype: float
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.area
+        """
+        return np.pi / np.sqrt(np.linalg.det(self.E))
+
+    @property
     def polynomial(self):
-        pass
+        r"""
+        Return ellipse as a polynomial
 
-    def plot(self) -> None:
-        pass
+        :return: polynomial
+        :rtype: ndarray(6)
+
+        An ellipse can be described by :math:`\vec{e} \in \mathbb{R}^6` which are the
+        coefficents of a quadratic in :math:`x` and :math:`y`
+
+        .. math::
+
+            e_0 x^2 + e_1 y^2 + e_2 xy + e_3 x + e_4 y + e_5 = 0
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.polynomial
+
+        :seealso: :meth:`Polynomial`
+        """
+        a = self._E[0, 0]
+        b = self._E[1, 1]
+        c = self._E[0, 1]
+        x_0, y_0 = self._centre
+
+        return np.array(
+            [
+                a,
+                b,
+                2 * c,
+                -2 * a * x_0 - 2 * c * y_0,
+                -2 * b * y_0 - 2 * c * x_0,
+                a * x_0**2 + b * y_0**2 + 2 * c * x_0 * y_0,
+            ]
+        )
+
+    def plot(self, **kwargs) -> None:
+        """
+        Plot ellipse
+
+        :param kwargs: arguments passed to :func:`~spatialmath.base.graphics.plot_ellipse`
+        :return: list of artists
+        :rtype: _type_
+
+        Example::
+
+            >>> from spatialmath import Ellipse
+            >>> from spatialmath.base import plotvol2
+            >>> plotvol2(5)
+            >>> e = Ellipse(E=np.array([[1, 1], [1, 2]]))
+            >>> e.plot()
+            >>> e.plot(filled=True, color='r')
 
 
-# alpha, beta, gamma, xc, yc, e0, e1, e2, e3, e4, e5 = symbols("alpha, beta, gamma, xc, yc, e0, e1, e2, e3, e4, e5")
-# solve(eq, [alpha, beta, gamma, xc, yc])
-# eq = [
-#     alpha - e0,
-#     beta- e1,
-#     2 * gamma - e2,
-#     -2 * (alpha * xc + gamma * yc) - e3,
-#     -2 * (beta * yc + gamma * xc) - e4,
-#     alpha * xc**2 + beta * yc**2 + 2 * gamma * xc * yc - 1 - e5
-# ]
+        .. plot::
+
+            from spatialmath import Ellipse
+            from spatialmath.base import plotvol2
+            ax = plotvol2(5)
+            e = Ellipse(E=np.array([[1, 1], [1, 2]]))
+            e.plot()
+            ax.grid()
+
+        .. plot::
+
+            from spatialmath import Ellipse
+            from spatialmath.base import plotvol2
+            ax = plotvol2(5)
+            e = Ellipse(E=np.array([[1, 1], [1, 2]]))
+            e.plot(filled=True, color='r')
+            ax.grid()
+
+        :seealso: :func:`~spatialmath.base.graphics.plot_ellipse`
+        """
+        return plot_ellipse(self._E, centre=self._centre, **kwargs)
+
+    def contains(self, p):
+        """
+        Test if points are contained by ellipse
+
+        :param p: point or points to test
+        :type p: arraylike(2), ndarray(2,N)
+        :return: true if point is contained within ellipse
+        :rtype: bool or list(bool)
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.contains((3,4))
+            >>> e.contains((0,0))
+
+        """
+        inside = []
+        p = smb.getmatrix(p, (2, None))
+        for x in p.T:
+            x -= self._centre
+            inside.append(np.linalg.norm(x.T @ self._E @ x) <= 1)
+
+        if len(inside) == 1:
+            return inside[0]
+        else:
+            return inside
+
+    def points(self, resolution=20) -> Points2:
+        """
+        Generate perimeter points
+
+        :param resolution: number of points on circumferance, defaults to 20
+        :type resolution: int, optional
+        :return: set of perimeter points
+        :rtype: Points2
+
+        Return a set of ``resolution`` points on the perimeter of the ellipse.  The perimeter
+        set is not closed, that is, last point != first point.
+
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.points()[:,:5]  # first 5 points
+
+        :seealso: :meth:`polygon` :func:`~spatialmath.base.graphics.ellipse`
+        """
+        return smb.ellipse(self.E, self.centre, resolution=resolution)
+
+    def polygon(self, resolution=10) -> Polygon2:
+        """
+        Approximate with a polygon
+
+        :param resolution: number of polygon vertices, defaults to 10
+        :type resolution: int, optional
+        :return: a polygon approximating the ellipse
+        :rtype: :class:`Polygon2` instance
+
+        Return a polygon instance with ``resolution`` vertices.  A :class:`Polygon2`` can be
+        used for intersection testing with lines or other polygons.
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from spatialmath import Ellipse
+            >>> e = Ellipse(radii=(1,2), centre=(3,4), theta=0.5)
+            >>> e.polygon()
+
+        :seealso: :meth:`points`
+        """
+        return Polygon2(smb.ellipse(self.E, self.centre, resolution=resolution - 1))
+
 
 if __name__ == "__main__":
-
     pass
     # print(Ellipse((500, 500), (100, 200)))
     # p = Polygon2([(1, 2), (3, 2), (2, 4)])
