@@ -298,8 +298,8 @@ class Twist3(BaseTwist):
     algebra se(3) of the corresponding SE(3) matrix.
 
     :References:
-        - **Robotics, Vision & Control**, Corke, Springer 2017.
-        - **Modern Robotics, Lynch & Park**, Cambridge 2017
+        - Robotics, Vision & Control for Python, Section 2.3.2.3, P. Corke, Springer 2023.
+        - Modern Robotics, Lynch & Park, Cambridge 2017
 
     .. note:: Compared to Lynch & Park this module implements twist vectors
         with the translational components first, followed by rotational
@@ -960,7 +960,7 @@ class Twist3(BaseTwist):
             >>> S = Twist3(T)
             >>> S.line()
         """
-        return Line3([Line3(-tw.v - tw.pitch * tw.w, tw.w) for tw in self])
+        return Line3([Line3(-tw.v + tw.pitch * tw.w, tw.w) for tw in self])
 
     @property
     def pole(self):
@@ -1008,7 +1008,7 @@ class Twist3(BaseTwist):
 
         theta = smb.getunit(theta, unit)
 
-        if smb.isscalar(theta):
+        if len(theta) == 1:
             # theta is a scalar
             return SE3(smb.trexp(self.S * theta))
         else:
@@ -1064,7 +1064,7 @@ class Twist3(BaseTwist):
         """
         from spatialmath.pose3d import SE3
 
-        theta = np.r_[smb.getunit(theta, unit)]
+        theta = smb.getunit(theta, unit)
 
         if len(self) == 1:
             return SE3([smb.trexp(self.S * t) for t in theta], check=False)
@@ -1269,8 +1269,8 @@ class Twist2(BaseTwist):
               moment vector (1 element) and direction vector (2 elements).
 
         :References:
-            - **Robotics, Vision & Control**, Corke, Springer 2017.
-            - **Modern Robotics, Lynch & Park**, Cambridge 2017
+            - Robotics, Vision & Control for Python, Section 2.2.2.4, P. Corke, Springer 2023.
+            - Modern Robotics, Lynch & Park, Cambridge 2017
 
         .. note:: Compared to Lynch & Park this module implements twist vectors
             with the translational components first, followed by rotational
@@ -1524,12 +1524,9 @@ class Twist2(BaseTwist):
         if unit != "rad" and self.isprismatic:
             print("Twist3.exp: using degree mode for a prismatic twist")
 
-        if theta is None:
-            theta = 1
-        else:
-            theta = smb.getunit(theta, unit)
+        theta = smb.getunit(theta, unit)
 
-        if smb.isscalar(theta):
+        if len(theta) == 1:
             return SE2(smb.trexp2(self.S * theta))
         else:
             return SE2([smb.trexp2(self.S * t) for t in theta])
@@ -1560,7 +1557,7 @@ class Twist2(BaseTwist):
         else:
             return [smb.skewa(x.S) for x in self]
 
-    def exp(self, theta=None, unit="rad"):
+    def exp(self, theta=1, unit="rad"):
         r"""
         Exponentiate a 2D twist
 
@@ -1595,12 +1592,14 @@ class Twist2(BaseTwist):
         """
         from spatialmath.pose2d import SE2
 
-        if theta is None:
-            theta = 1.0
-        else:
-            theta = smb.getunit(theta, unit)
+        theta = smb.getunit(theta, unit)
 
-        return SE2(smb.trexp2(self.S * theta))
+        if len(self) == 1:
+            return SE2([smb.trexp2(self.S * t) for t in theta], check=False)
+        elif len(self) == len(theta):
+            return SE2([smb.trexp2(s * t) for s, t in zip(self.S, theta)], check=False)
+        else:
+            raise ValueError("length mismatch")
 
     def unit(self):
         """
